import { mergeIconSets, importFromFigma } from '@iconify/tools';
import consola from 'consola';
import { COLORED_POSTFIX, optimizeIconSet, normalizeColoredName } from '../../helpers/icon-set.mjs';
import 'cross-fetch/polyfill';

async function importFigmaIconSets(options) {
  const {
    files,
    token,
    cache = false,
    cacheOptions,
    preserveColorsGroup
  } = options;
  if (typeof token !== "string") {
    throw new TypeError("token type error");
  }
  if (!token) {
    throw new Error("token not found");
  }
  async function importFigmaFile(file) {
    const { id, pages, nodeIds, prefix } = file;
    const result = await importFromFigma({
      prefix,
      file: id,
      ids: nodeIds,
      pages,
      token,
      cacheDir: cache ? `.figma-cache/${prefix}` : void 0,
      ...cacheOptions,
      /** Support node type: 'FRAME' | 'COMPONENT' | 'INSTANCE' */
      iconNameForNode: (node) => {
        if (!["COMPONENT", "INSTANCE", "FRAME"].includes(node.type)) {
          return;
        }
        if (node.name.startsWith(`${prefix}-`)) {
          const nameRegExp = /^[a-z]([a-z0-9\-])*[a-z0-9]$/;
          if (!nameRegExp.test(node.name)) {
            throw new Error(
              `Unexpected icon name: ${node.name}, regexp: ${nameRegExp}`
            );
          }
          const newName = node.name.replace(`${prefix}-`, "");
          const unexpectedHyphensRegExp = /-{2,}/;
          if (unexpectedHyphensRegExp.test(newName)) {
            throw new Error(
              `Unexpected icon name: ${node.name}, contains duplicate hyphens`
            );
          }
          if (preserveColorsGroup && node.parents.some((item) => {
            return ["FRAME", "GROUP", "SECTION"].includes(item.type) && item.name.trim() === preserveColorsGroup;
          })) {
            consola.log("colored icon", node.name);
            return `${newName}${COLORED_POSTFIX}`;
          }
          consola.log("icon", node.name);
          return newName;
        }
      }
    });
    if (typeof result === "string") {
      return result;
    }
    const iconSet = result.iconSet;
    optimizeIconSet(iconSet, {
      preserveColors: ({ iconName }) => {
        return iconName.endsWith(COLORED_POSTFIX);
      }
    });
    iconSet.forEach((name) => {
      if (name.endsWith(COLORED_POSTFIX)) {
        iconSet.rename(name, normalizeColoredName(name));
      }
    });
    return iconSet;
  }
  const iconSetsResult = await Promise.allSettled(
    files.map(async (item) => {
      return await importFigmaFile(item);
    })
  );
  const okIconSets = [];
  for (const [index, item] of iconSetsResult.entries()) {
    if (item.status === "fulfilled") {
      if (item.value === "not_modified") {
        consola.log(`file id: ${files[index].id} not modified.`);
        continue;
      }
      okIconSets.push(item.value);
      consola.log(`file id: ${files[index].id} import success.`);
    } else {
      consola.error(`file id: ${files[index].id} import failed:`, item.reason);
    }
  }
  consola.log(
    `Icon sets imported: ${okIconSets.length}, failed: ${files.length - okIconSets.length}`
  );
  const iconSetsMap = okIconSets.reduce((previous, current) => {
    return {
      ...previous,
      [current.prefix]: current.prefix in previous ? [...previous[current.prefix], current] : [current]
    };
  }, {});
  const mergedIconSets = [];
  Object.keys(iconSetsMap).forEach((prefix) => {
    const prefixIconSets = iconSetsMap[prefix];
    if (prefixIconSets.length === 1) {
      mergedIconSets.push(iconSetsMap[prefix][0]);
      return;
    }
    mergedIconSets.push(
      prefixIconSets.slice(1).reduce((previous, current) => {
        return mergeIconSets(previous, current);
      }, prefixIconSets[0])
    );
  });
  if (mergedIconSets.length !== okIconSets.length) {
    consola.log(
      `Merged icon sets from ${okIconSets.length} to ${mergedIconSets.length}`
    );
  }
  return mergedIconSets;
}

export { importFigmaIconSets };
