'use strict';

Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });

const tools = require('@iconify/tools');
const attribs = require('@iconify/tools/lib/colors/attribs');
const utils = require('@iconify/utils');
const icons = require('@iconify/utils/lib/css/icons');
const fse = require('fs-extra');
const dynamic = require('../dynamic.cjs');
const loader = require('../loader.cjs');

const COLORED_POSTFIX = "__colored";
function normalizeColoredName(name) {
  return name.replace(COLORED_POSTFIX, "");
}
function optimizeIconSet(iconSet, options = {}) {
  const { preserveColors } = options;
  iconSet.forEachSync((name, type) => {
    if (type !== "icon") {
      return;
    }
    const svg = iconSet.toSVG(name);
    if (!svg) {
      iconSet.remove(name);
      return;
    }
    try {
      tools.cleanupSVG(svg);
      tools.parseColors(svg, {
        defaultColor: "currentColor",
        callback: (attr, colorString, parsedColor, tagName, item) => {
          if (attribs.specialColorAttributes.includes(attr) || preserveColors && !!preserveColors({
            iconName: name,
            attr,
            colorString,
            parsedColor,
            tagName,
            item
          })) {
            return colorString;
          }
          const result = !parsedColor || tools.isEmptyColor(parsedColor) ? colorString : "currentColor";
          return result;
        }
      });
      tools.runSVGO(svg, {
        cleanupIDs: (id) => {
          return `${normalizeColoredName(name)}_${id}`;
        }
      });
    } catch (err) {
      console.error(`Error parsing ${name}:`, err);
      iconSet.remove(name);
      return;
    }
    iconSet.fromSVG(name, svg);
  });
}
function getIconSetIconStyles(iconSetName, options) {
  const { pluginOptions, staticIconNames = "*" } = options;
  const values = {};
  const iconSetJson = loader.ensureLoadIconSet(iconSetName, pluginOptions);
  const iconSet = new tools.IconSet(iconSetJson);
  if (!staticIconNames) {
    return values;
  }
  iconSet.forEach((name) => {
    if (Array.isArray(staticIconNames) && !staticIconNames.includes(name)) {
      return;
    }
    const generated = icons.getIconsCSSData(iconSetJson, [name], {
      iconSelector: ".icon"
    });
    values[`${iconSetName}--${name}`] = dynamic.transformCSSDataToRules(
      generated,
      pluginOptions
    );
  });
  return values;
}
function loadIconifyJsonPath(path) {
  try {
    if (fse.existsSync(path)) {
      const rawData = fse.readJsonSync(path, "utf8");
      const validatedData = utils.validateIconSet(rawData);
      return new tools.IconSet(validatedData);
    }
  } catch (err) {
    console.log(`loadIconifyJsonPath ${path} failed:`, err);
  }
  return null;
}

exports.COLORED_POSTFIX = COLORED_POSTFIX;
exports.getIconSetIconStyles = getIconSetIconStyles;
exports.loadIconifyJsonPath = loadIconifyJsonPath;
exports.normalizeColoredName = normalizeColoredName;
exports.optimizeIconSet = optimizeIconSet;
