import { cleanupSVG, parseColors, isEmptyColor, runSVGO, IconSet } from '@iconify/tools';
import { specialColorAttributes } from '@iconify/tools/lib/colors/attribs';
import { validateIconSet } from '@iconify/utils';
import { getIconsCSSData } from '@iconify/utils/lib/css/icons';
import fse from 'fs-extra';
import { transformCSSDataToRules } from '../dynamic.mjs';
import { ensureLoadIconSet } from '../loader.mjs';

const COLORED_POSTFIX = "__colored";
function normalizeColoredName(name) {
  return name.replace(COLORED_POSTFIX, "");
}
function optimizeIconSet(iconSet, options = {}) {
  const { preserveColors } = options;
  iconSet.forEachSync((name, type) => {
    if (type !== "icon") {
      return;
    }
    const svg = iconSet.toSVG(name);
    if (!svg) {
      iconSet.remove(name);
      return;
    }
    try {
      cleanupSVG(svg);
      parseColors(svg, {
        defaultColor: "currentColor",
        callback: (attr, colorString, parsedColor, tagName, item) => {
          if (specialColorAttributes.includes(attr) || preserveColors && !!preserveColors({
            iconName: name,
            attr,
            colorString,
            parsedColor,
            tagName,
            item
          })) {
            return colorString;
          }
          const result = !parsedColor || isEmptyColor(parsedColor) ? colorString : "currentColor";
          return result;
        }
      });
      runSVGO(svg, {
        cleanupIDs: (id) => {
          return `${normalizeColoredName(name)}_${id}`;
        }
      });
    } catch (err) {
      console.error(`Error parsing ${name}:`, err);
      iconSet.remove(name);
      return;
    }
    iconSet.fromSVG(name, svg);
  });
}
function getIconSetIconStyles(iconSetName, options) {
  const { pluginOptions, staticIconNames = "*" } = options;
  const values = {};
  const iconSetJson = ensureLoadIconSet(iconSetName, pluginOptions);
  const iconSet = new IconSet(iconSetJson);
  if (!staticIconNames) {
    return values;
  }
  iconSet.forEach((name) => {
    if (Array.isArray(staticIconNames) && !staticIconNames.includes(name)) {
      return;
    }
    const generated = getIconsCSSData(iconSetJson, [name], {
      iconSelector: ".icon"
    });
    values[`${iconSetName}--${name}`] = transformCSSDataToRules(
      generated,
      pluginOptions
    );
  });
  return values;
}
function loadIconifyJsonPath(path) {
  try {
    if (fse.existsSync(path)) {
      const rawData = fse.readJsonSync(path, "utf8");
      const validatedData = validateIconSet(rawData);
      return new IconSet(validatedData);
    }
  } catch (err) {
    console.log(`loadIconifyJsonPath ${path} failed:`, err);
  }
  return null;
}

export { COLORED_POSTFIX, getIconSetIconStyles, loadIconifyJsonPath, normalizeColoredName, optimizeIconSet };
